function [CR,mdat] = CRboundNKQ(specdat,zAs)
% function [CR,mdat] = CRboundNKQ(specdat,zAs)
% computes Cramer-Rao lower bound matrices for specified zAs thicknesses. 
%  inputs:
%    specdat: spectrum struct with fields
%       specnum: number spectrum
%       zmus: mus_1(E) + 1i*mus_2(E)
%       idx_threshold: the indices for interbin energies
%    zAs: As(:,1) + 1i*As(:,2)
%  outputs:
%    CR: an nbins+1,nbins+1,numel(zAs) 3D matrix of Cramer Rao lower bounds
%    mdat: debugging info struct. See code for fields
%  REA 5/29-6/27/10
% 6/27/10 add NQ CR bound and M matrices. Output in mdat


if isfield(specdat,'Ethreshold')
   nbins = numel(specdat.Ethreshold) + 1;
   idx_threshold = zeros(nbins-1,1);
   for k = 1:(nbins-1)
      [minval,idx] = min(abs(specdat.Ethreshold(k) - specdat.egys));
      idx_threshold(k) = idx;
   end
   specdat.idx_threshold = idx_threshold;
end

if ~isfield(specdat,'zmus') && isfield(specdat,'mus')
    specdat.zmus = zcomp(specdat.mus);
else
    error('specdat mus have mus or zmus field');
end

       % compute the indexes into the egy array for the N bins
idx_threshold = specdat.idx_threshold;       
nbins = numel(idx_threshold)+1;       
bin_edges = [0; idx_threshold(:);numel(specdat.egys)];
idxs = cell(nbins,1);
for k = 1:nbins
    idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
end
nthicknesses = numel(zAs);

CR= zeros(2,2,nthicknesses); % Cramer-Rao bound for A variances: 2x2
CR_NQ = zeros(2,2,nthicknesses); 
CRinv= zeros(2,2,nthicknesses); % inverse of Cramer-Rao bound for A variances: 2x2
CR_NQinv = zeros(2,2,nthicknesses); 
M_all = zeros(nbins+1,2,nthicknesses); % the effective mus at each thickness
M_all_NQ = zeros(2,2,nthicknesses); % the effective mus at each thickness for NQ detector
RL_all = zeros(nbins+1,nbins+1,nthicknesses); % the covariance at each thickness
RL_all_NQ = zeros(2,2,nthicknesses); % the covariance at each thickness for NQ detector
lambdas_all = zeros(nthicknesses,nbins);
Ebar_all= zeros(nthicknesses,1);
Ebars_all= zeros(nthicknesses,nbins);
E2bar_all= zeros(nthicknesses,1);
Qs_all = zeros(nthicknesses,1);

specnum_all = specdat.specnum(:);
zmus_all = specdat.zmus(:);
egys_all = specdat.egys(:);
dE = egys_all(2)-egys_all(1);

for kt = 1:nthicknesses
        % compute the data needed for CR matrix: M and covariance of log signal
      zA = zAs(kt);
      M = zeros(nbins+1,2);
      lambdas = zeros(nbins,1);
      Ebars = zeros(nbins,1);
          % overall transmitted spectrum
      Transmission_E_all = exp(-zcomp(zmus_all)*zcomp(zA)');
      spec_transmitted_all = specnum_all.*Transmission_E_all;
      sp_norm_all = spec_transmitted_all/(sum(spec_transmitted_all)*dE);
      for kbin = 1:nbins
          idx = idxs{kbin};
          egys = egys_all(idx);
          zmus =zmus_all(idx);

          spec_transmitted = spec_transmitted_all(idx);
          lambdas(kbin) = sum(spec_transmitted)*dE;
          sp_norm = spec_transmitted(:)/ lambdas(kbin);
             % compute M for counting signals
          mz = sum(zmus.*sp_norm)*dE;
          M(kbin,:) = zcomp(mz);
             % Ebar and lambda for each bin
          Ebars(kbin) = sum(egys.*sp_norm)*dE;
      end
    lambdas_all(kt,:) = lambdas(:)';
    Ebars_all(kt,:) = Ebars';
       % last row of M is for integrated energy of complete spectrum
    sp_egy_transmitted_all = spec_transmitted_all.*egys_all;
    sp_egy_norm = sp_egy_transmitted_all/(sum(sp_egy_transmitted_all)*dE);
    mQz = sum(zmus_all.*sp_egy_norm);
    M(nbins+1,:) = zcomp(mQz);
    M_all(:,:,kt) = M;
       % do NQ calcs
    M_NQ = zeros(2);
    mNz = sum(zmus_all.*sp_norm_all);
    M_NQ(1,:) = zcomp(mNz);
    M_NQ(2,:) = zcomp(mQz);
      % covariance matrix of log signal
           % for the covariance of log signal see last section of Appendix B of 
           % my CDpaper3.pdf (latest version of 'Count Det paper')
           % see also my paper lab notebook Aug 14-15, 2009 and Onenote (search NKQ)
    RL = zeros(nbins+1); % square matrix
    lambda = sum(lambdas);
    RL(1:nbins,1:nbins) = diag(lambda./lambdas); 
    Ebar_all(kt) = sum(egys_all.*sp_norm_all);
    RL(1:nbins,nbins+1)= Ebars./Ebar_all(kt);
    RL(nbins+1,1:nbins)= Ebars'./Ebar_all(kt);
    E2bar_all(kt) = sum((egys_all.^2) .* sp_norm_all);
    F = E2bar_all(kt)/(Ebar_all(kt)^2); % F for overall spectrum
    RL(nbins+1,nbins+1) = F;
    RL = RL/lambda;
    RL_all(:,:,kt) = RL;
    Qs_all(kt) = lambda*Ebar_all(kt);
    CRinv(:,:,kt) = (M'*inv(RL)*M); % the A covariance Cramer-Rao bound
    CR(:,:,kt) = inv(CRinv(:,:,kt)); % the A covariance Cramer-Rao bound
       % NQ calcs
    RL_NQ = ones(2); % 2x2
    RL_NQ(2,2) = F;
    RL_NQ = RL_NQ/lambda;
    CR_NQinv(:,:,kt)= (M_NQ'*inv(RL_NQ)*M_NQ);
    CR_NQ(:,:,kt)= inv(CR_NQinv(:,:,kt));
end
mdat = [];
if nargout>1
      % compute stats for last thickness. Most likely these stats will be
      % used for single thickness
   mdat.M_all = M_all;
   mdat.RL_N2Q = RL_all;
   mdat.RL_NQ = RL_NQ;
   M_NK = M(1:nbins,:);
   RL_NK = RL(1:nbins,1:nbins);
   mdat.M_NK = M_NK;
   mdat.RL_NK = RL_NK;
   mdat.CR_NKinv = (M_NK'*inv(RL_NK)*M_NK);
   mdat.CR_NK = inv(mdat.CR_NKinv);
   mdat.lambdas = lambdas_all;
   mdat.lambda = sum(mdat.lambdas);
   mdat.Qs = Qs_all;
   mdat.Ebar = Ebar_all;
   mdat.Ebars = Ebars_all;
   mdat.E2bar = E2bar_all;
   mdat.F = F;
   mdat.M_NQ = M_NQ;
   mdat.CR_NQinv = CR_NQinv;
   mdat.CR_NQ = CR_NQ;
   mdat.dE = dE;
   mdat.specnorm = sp_norm_all;
   mdat.spec_transmitted = spec_transmitted_all;
   mdat.CR_N2Qinv = CRinv;
   mdat.CR_N2Q = CR;
end   