function stats = OptimalSNR(specdat,varargin)
%  function stats = OptimalSNR(specdat,varargin)
%  Compute SNR with different types of detector for specified spectrum
%  SNR is normalized by dividing by ideal, full spectrum SNR
%  Detectors are: N, Q, NQ, N2, and N2Q
%  inputs:
%      specdat: a struct with the x-ray spectrum and attenuation data. Must have fields
%          specnum: the photon number spectrum
%          egys: (keV) the corresponding x-ray energies.
%          mus: a (numel(egys),2) array with the attenuation coefficients of the 2 basis materials along columns
%                basis materials are assumed to be background, feature (columns 1 and 2)
%      dA: (optional=[1;-1]) The difference  of the background material and feature A vectors length 2 vector
%      t_b: (optional=0) The thickness of the background material (column 1 of mus) 
%      f2keep: (optional = 0) clip off parts of attenuated spectrum tail and head that are below this value*max(specnum)
%  outputs:
%      stats: a struct with fields
%  26/Mar/2013 10:49
%  COPYRIGHT:  Robert E. Alvarez, 2013. All rights reserved.

nreqargs = 1;

   % validate input argument
assert(isstruct(specdat) && isfield(specdat,'egys') && isfield(specdat,'specnum') && isfield(specdat,'mus') );
assert(numel(specdat.egys) == numel(specdat.specnum) );
mus_size = size(specdat.mus);
assert((mus_size(1) == numel(specdat.egys)) && (mus_size(2) == 2) );

t_b = 0;
f2keep = 0;
dA = [1;-1];
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'da';         dA=varargin{i+1};   assert(isvector(dA) && numel(dA)==2);   i=i+1;
     case 't_b';         t_b=varargin{i+1};   assert(isscalar(t_b)); assert(t_b>=0);  i=i+1;
     case 'f2keep';         f2keep=varargin{i+1};   assert(isscalar(f2keep)); assert(f2keep>=0);  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

     % attenuate by background material
specnum = specdat.specnum(:).*exp(-t_b*specdat.mus(:,1));
     % clip off near zero parts, else makes trouble with bins with near zero points
clip_from_tail = find(specnum > f2keep*max(specnum),1,'first'); 
clip_from_head = find(specnum > f2keep*max(specnum),1,'last'); 
idxs2keep = clip_from_tail:clip_from_head;
sptran.egys = specdat.egys(idxs2keep);
sptran.specnum = specnum(idxs2keep);
sptran.mus = specdat.mus(idxs2keep,:);
sptran.negys = numel(sptran.egys);
sptran.dmus = diff(sptran.mus,1,2); 

   % find optimal bin separation idx/energy
idxs4plot = 4:(sptran.negys-4); % indexes into sptran.egys array for interbin energies
nidxs = numel(idxs4plot);
d2s = zeros(nidxs,2); % columns are SNR2(N2 and N2Q )/SNR2ideal
for kidx = 1:nidxs
    sptran.idx_threshold = idxs4plot(kidx);
    [CR,mdat] = CRboundNKQ(sptran,t_b);
    spnorm = mdat.spec_transmitted/mdat.lambda;
    d2ideal = mdat.lambda*trapz(sptran.egys,mdat.specnorm.*(sptran.dmus.^2)); 
    % do not divide by 2 or multiply by tf^2 since will do ratio
    d2s(kidx,1) = (dA'*mdat.CR_NKinv*dA)/d2ideal; % N2 -- do not divide by 2 since will do ratio
    d2s(kidx,2) = (dA'*mdat.CR_N2Qinv*dA)/d2ideal; % N2Q     "       "
end

stats.d2ideal = d2ideal/2;  % did not divide by 2 so do it now
stats.d2NQ = (dA'*mdat.CR_NQinv*dA)/2; %  divide by 2 per definition
        % N2 is first column of d2s
[d2max,idx] = max(d2s(:,1));
stats.d2N2 = d2max*d2ideal/2;
stats.idx_threshold_N2 = idxs4plot(idx);
stats.Ethreshold_N2 = sptran.egys(stats.idx_threshold_N2);
        % N2Q is 2nd column of d2s
[d2max,idx] = max(d2s(:,2));
stats.d2N2Q = d2max*d2ideal/2;
stats.idx_threshold_N2Q = idxs4plot(idx);
stats.Ethreshold_N2Q = sptran.egys(stats.idx_threshold_N2Q);
        % N only
stats.d2N = (mdat.lambda/2)*(trapz(sptran.egys,spnorm(:).*sptran.dmus))^2;        
        % Q only
specegy = sptran.egys(:).*spnorm;
specegy_norm = specegy/trapz(sptran.egys,specegy);        
stats.d2Q = (mdat.lambda/(2*mdat.F))*(trapz(sptran.egys,specegy_norm(:).*sptran.dmus))^2;    
stats.idxs4plot = idxs4plot;
stats.egys4plot = sptran.egys(idxs4plot);
stats.d2s = d2s;  
stats.sptran = sptran;  