% P35OptVsKV.m Optimal SNR vs. tube voltage
%
% Functions used in  Post  P35OptVsKV
% -----------------
%  OptimalSNR.m
%  CRboundNKQ.m
%  ARROW.M
%  exportfig.m
%  BodyMaterialCompositionFunc.m
%  ChemicalSymbols2AtomicNumbers.m
%  ParseChemicalFormula.m
%  XrayData.m
%  XrayMu.m
%  XrayTubeSpectrumTasmip.m
%  AddStruct2StructArray.m
%  zcomp.m
% -----------------
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2013. All rights reserved.
 
%% init vars
post_title = 'P35OptVsKV';
dirname = ['G:\Projects\Blog\Posts\' post_title '\'];
addpath(dirname);
addpath('G:\Projects\DEsolvePaper\Code\');
addpath('G:\Projects\CountDetPhysics\PileupSim\');
addpath([dirname '\code\']);

%% Set up the information to compute the basis functions: 
  %  the background material is soft tissue and the feature is cortical bone
dat = BodyMaterialCompositionFunc;
material_back = 'soft_tissue';
ztback = dat.(material_back)(2:end,:);% tables with atomic number and fraction by weight
material_feat = 'bone_cortical'; % 'adipose'; %
ztfeat = dat.(material_feat)(2:end,:);% for background and feature materials 
zAback = 3 + 0i; % background thickness: 3 gm/cm^2 soft tissue and no bone
tf = 1; % thickness of feature
dA = [1; -1]; % tf*DA is difference in A-vector for background and feature

%% get d2s vs. kV at fixed thickness
t_b = 3; % gm/cm^2 of background material
kVs = 40:120;
number_kVs = numel(kVs);
det_types = {'Q','N','NQ','N2','N2Q'};
ndet_types = length(det_types);
  % optimal threshold energies and indexes
stats_all_fixed_thick = struct([]); 
stcell = cell(number_kVs,1);
for k = 1:number_kVs
       % SNR^2 for this tube voltgae as a function of inter-bin energy
    kV = kVs(k);
   [dum,dum,specdat] = XrayTubeSpectrumTasmip(kV,'number_spectrum','clipzeros');
    specdat.mus = [xraymu(ztback,specdat.egys) xraymu(ztfeat,specdat.egys)];
        % get the data 
    stats = OptimalSNR(specdat,'t_b',t_b,'dA',dA); 
    stats.kV = kV;
    stats_all_fixed_thick =  AddStruct2StructArray(stats_all_fixed_thick,stats);
    stcell{k} = stats;
end

%% SNR vs. interbin threshold energy for N2 and N2Q
   % find the nearest data from prev cell to kVs I want to plot
kVs2plot = [40 70 90 120];
ncases2plot = numel(kVs2plot);
kVs_all = [stats_all_fixed_thick.kV]; % all the kVs for the data in previous cell
idxs2use = interp1(kVs_all,1:numel(kVs_all),kVs2plot,'nearest'); % indexes of kVs to plot
kVs2plot = kVs_all(idxs2use); % actual values since may not find exact match
   % do the plot
fh = figure;
pha_dets_fieldnames = {'N2','N2Q'};
kplot = 1;
colors = 'kr';
linespecs2 = {'-',':'};

for k = 1:ncases2plot
        % get the data from data calculated in previous cell
    stats = stats_all_fixed_thick(idxs2use(kplot));
        % plot the data
    subplot(2,2,kplot);
    phs = zeros(2,1); % handles to plots
    egys = stats.egys4plot;
    hold on
    for kdet = 1:2 % N2, N2Q are detectors
        d2rel = stats.d2s(:,kdet);
        phs(kdet) = plot(egys,d2rel,[linespecs2{kdet} colors(kdet)],'linewidth',2);
           % draw and label line at maximum
        [d2max,idx] = max(d2rel);
           %  offset line and text so they do not merge
        if kdet==2
            line([egys(idx),egys(idx)],[d2max + 0.01 ,d2max + 0.08],'color',colors(kdet),'linewidth',2)
            text(egys(idx)-6,d2rel(idx) + 0.15,[num2str(egys(idx),'%3.0f')],'fontsize',14,'color',colors(kdet));  
         else
            line([egys(idx),egys(idx)],[d2max - 0.08 ,d2max - 0.01],'color',colors(kdet),'linewidth',2)
            text(egys(idx)-6,d2rel(idx) - 0.15,[num2str(egys(idx),'%3.0f')],'fontsize',14,'color',colors(kdet));  
        end        
    end
    hold off
    text(kVs2plot(kplot)/4,0.1,[sprintf('%3.0f',kVs2plot(kplot)) ' kVp'],'fontsize',14);
    if kplot==1 
         hl = legend(phs,pha_dets_fieldnames,'fontsize',14,'location','West');
    end
    legend(gca,'boxoff');
    set(gca,'fontsize',14,'linewidth',1.5);
    axis([0 (kVs2plot(kplot)+5) 0 1.1])
    if mod(kplot,2)==1
      ylabel('SNR^2/SNR^2_i_d_e_a_l','fontsize',14);
    end   
    kplot = kplot + 1;
end
psn1 = [50 850 475 500];
set(fh,'color','w','Colormap',[0 0 0],'Position',psn1)
ht = annotation('textbox',[0.35 0.075 0.6 0],'string','bin separation energy (keV)','Fontsize',14, ...
       'backgroundcolor','w','edgecolor','w');

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRvsEthreshold';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% get data for variation of threshold energy vs. object thickness at fixed kV
nthicknesses = 30;
t_bs = linspace(0,30,nthicknesses); % gm/cm^2 of background material
kV = 120;
[dum,dum,specdat] = XrayTubeSpectrumTasmip(kV,'number_spectrum','clipzeros');
specdat.mus = [xraymu(ztback,specdat.egys) xraymu(ztfeat,specdat.egys)];
  % optimal threshold energies and indexes
stats_all_fixed_kV = struct([]); 
for k = 1:nthicknesses
       % SNR^2 for this tube voltgae as a function of inter-bin energy
        % get the data 
    stats = OptimalSNR(specdat,'t_b',t_bs(k),'dA',dA); 
    stats.kV = kV;
    stats_all_fixed_kV =  AddStruct2StructArray(stats_all_fixed_kV,stats);
end

%% plot optimal threshold energy vs. tube voltage
fh2 = figure;
   % get data from stats_all_fixed_thick and stats_all_fixed_kV
subplot(1,2,1);
egys_maxN2Q = [stats_all_fixed_thick.Ethreshold_N2Q];
egys_maxN2 = [stats_all_fixed_thick.Ethreshold_N2];
hold on
plot(kVs,egys_maxN2Q,'-k','linewidth',2);
plot(kVs,egys_maxN2,'--r','linewidth',2);
hold off
v = axis;
axis([v(1:2) 0 70]); % include y-axis origin since % change important
legend({'N2Q','N2'},'location','southwest');
set(gca,'fontsize',14,'linewidth',1.5);
xlabel('tube voltage (kV)','fontsize',14);
ylabel('optimal bin sep. energy (keV)','fontsize',14);

subplot(1,2,2)
egys_maxN2Q = [stats_all_fixed_kV.Ethreshold_N2Q];
egys_maxN2 = [stats_all_fixed_kV.Ethreshold_N2];
hold on
plot(t_bs,egys_maxN2Q,'-k','linewidth',2);
plot(t_bs,egys_maxN2,'--r','linewidth',2);
hold off
v = axis;
axis([v(1:2) 0 70]); % include y-axis origin since % change important
set(gca,'fontsize',14,'linewidth',1.5);
xlabel('object thickness(gm/cm^2)','fontsize',14);

set(fh2,'color','w','Colormap',[0 0 0],'position',[358        1204         500         307]);

%% export the figure as png and eps
figno = sprintf('-f%d',fh2);
fname = 'OptimalEthresholdvsKVthickness';
   % export as eps
exportfig(fh2,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot d2s vs. kV at fixed thickness
linespecs = {'-','-*','-.','--','-o'};
linespecs = fliplr(linespecs);
colors = 'krbgm';
fh3 = figure;
hold on
for kdet = ndet_types:-1:1
       % get data from stats_all_fixed_thick data calculated  3 cells up
    d2rel = [];
    for k = 1:number_kVs
        d2rel(k) = stats_all_fixed_thick(k).(['d2' det_types{kdet}])/stats_all_fixed_thick(k).d2ideal;
    end
    plot(kVs,d2rel,[linespecs{kdet} colors(kdet)],'linewidth',1.5);
end
hl = legend(det_types(end:-1:1),'location','southwest');
xlabel('tube voltage (kV)','fontsize',14);
ylabel('d^2/d^2_i_d_e_a_l','fontsize',14);
text(60,0.65,'object thickness: 3 gm/cm^2','fontsize',14);
set(gca,'fontsize',14,'linewidth',1.5);
axis([min(kVs) max(kVs) 0 1.1]);
psn1 = [300 1150 450 380];
set(fh3,'color','w','Colormap',[0 0 0],'Position',psn1)

%% export the figure as png and eps
figno = sprintf('-f%d',fh3);
fname = 'd2vsKV';
   % export as eps
exportfig(fh3,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh3,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot d2 vs thickness at fixed kV
fh4 = figure;
hold on
for kdet = ndet_types:-1:1
       % get data from stats_all_fixed_thick data calculated  3 cells up
    d2rel = [];
    for k = 1:nthicknesses
        d2rel(k) = stats_all_fixed_kV(k).(['d2' det_types{kdet}])/stats_all_fixed_kV(k).d2ideal;
    end
    plot(t_bs,d2rel,[linespecs{kdet} colors(kdet)],'linewidth',1.5);
end
hl = legend(det_types(end:-1:1),'location','bestoutside');
xlabel('object thickness (gm/cm^2)','fontsize',14);
ylabel('d^2/d^2_i_d_e_a_l','fontsize',14);
text(10,0.05,'tube voltage: 120 kV','fontsize',14);
set(gca,'fontsize',14,'linewidth',1.5);
axis([min(t_bs) max(t_bs) 0 1.1]);
psn1 = [300 1150 650 380];
set(fh4,'color','w','Colormap',[0 0 0],'Position',psn1)

%% export the figure as png and eps
figno = sprintf('-f%d',fh4);
fname = 'd2vsThickness';
   % export as eps
exportfig(fh4,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh3,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot d2 vs thickness at fixed kV and fixed Ethreshold
fh4 = figure;
Ethresholds = [50 40]; % keV for N2 and N2Q
       % get data from stats_all_fixed_thick data calculated  3 cells up
d2rel = zeros(nthicknesses,2);
for k = 1:nthicknesses
    stats = stats_all_fixed_kV(k);
    d2s = stats.d2s;
    [dum,idxn2] = min(abs( stats.egys4plot - Ethresholds(1)));
    [dum,idxn2q] = min(abs( stats.egys4plot - Ethresholds(2)));
    d2rel(k,:) = [d2s(idxn2,1) d2s(idxn2q,2)];
end
hold on
plot(t_bs,d2rel(:,2),'-m','linewidth',1.5);
plot(t_bs,[stats_all_fixed_kV.d2N2Q]./[stats_all_fixed_kV.d2ideal], ...
    '--m','linewidth',1.5);
plot(t_bs,d2rel(:,1),'-g','linewidth',1.5);
plot(t_bs,[stats_all_fixed_kV.d2N2]./[stats_all_fixed_kV.d2ideal], ...
   '--g','linewidth',1.5);
hold off
hl = legend('N2Q','N2Qopt','N2','N2opt','location','bestoutside');
xlabel('object thickness (gm/cm^2)','fontsize',14);
ylabel('d^2/d^2_i_d_e_a_l','fontsize',14);
text(10,0.05,'tube voltage: 120 kV','fontsize',14);
set(gca,'fontsize',14,'linewidth',1.5);
axis([min(t_bs) max(t_bs) 0 1.1]);
psn1 = [300 1150 650 380];
set(fh4,'color','w','Colormap',[0 0 0],'Position',psn1)

%% export the figure as png and eps
figno = sprintf('-f%d',fh4);
fname = 'd2vsThicknessNonOptIdx';
   % export as eps
exportfig(fh4,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh4,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
